package com.zanxiang.game.back.serve.service.impl;

import cn.hutool.http.HttpStatus;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.advertising.tencent.base.AdvertisingTencentServer;
import com.zanxiang.advertising.tencent.base.pojo.dto.DataReportOfAppIdRpcDTO;
import com.zanxiang.advertising.tencent.base.pojo.dto.UserActionRpcDTO;
import com.zanxiang.advertising.tencent.base.rpc.IUserActionSetRpc;
import com.zanxiang.erp.base.ErpServer;
import com.zanxiang.erp.base.rpc.ISysUserRpc;
import com.zanxiang.erp.security.util.SecurityUtil;
import com.zanxiang.game.back.serve.dao.mapper.GameTencentUserMapper;
import com.zanxiang.game.back.serve.pojo.dto.GameTencentUserDTO;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentBackLog;
import com.zanxiang.game.back.serve.pojo.entity.GameTencentUser;
import com.zanxiang.game.back.serve.pojo.enums.ActionTypeEnum;
import com.zanxiang.game.back.serve.pojo.enums.BackStatusEnum;
import com.zanxiang.game.back.serve.pojo.vo.GameTencentUserVO;
import com.zanxiang.game.back.serve.service.IGameTencentBackLogService;
import com.zanxiang.game.back.serve.service.IGameTencentUserService;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 游戏腾讯用户表 服务实现类
 * </p>
 *
 * @author auto
 * @since 2023-06-01
 */
@Slf4j
@Service
public class GameTencentUserServiceImpl extends ServiceImpl<GameTencentUserMapper, GameTencentUser> implements IGameTencentUserService {
    @DubboReference(providedBy = AdvertisingTencentServer.SERVER_DUBBO_NAME)
    private IUserActionSetRpc userActionSetRpc;
    @DubboReference(providedBy = ErpServer.SERVER_DUBBO_NAME)
    private ISysUserRpc sysUserRpc;
    @Autowired
    private IGameTencentBackLogService gameTencentBackLogService;
    @Override
    public IPage<GameTencentUserVO> tencentUserLogList(GameTencentUserDTO dto) {
        return page(dto.toPage(), new LambdaQueryWrapper<GameTencentUser>()
                .eq(StringUtils.isNotBlank(dto.getChannel()), GameTencentUser::getChannel, dto.getChannel())
                .eq(null != dto.getGameId(), GameTencentUser::getGameId, dto.getGameId())
                .eq(null != dto.getAdAccountId(), GameTencentUser::getAdAccountId, dto.getAdAccountId())
                .eq(StringUtils.isNotBlank(dto.getWechatAppId()), GameTencentUser::getWechatAppId, dto.getWechatAppId())
                .eq(StringUtils.isNotBlank(dto.getWechatOpenid()), GameTencentUser::getWechatOpenid, dto.getWechatOpenid())
                .eq(null != dto.getIsBack(), GameTencentUser::getIsBack, dto.getIsBack())
                .orderByDesc(GameTencentUser::getCreateTime)
        ).convert(item -> {
            GameTencentUserVO copy = BeanUtil.copy(item, GameTencentUserVO.class);
            copy.setUpdateName(null == item.getUpdateBy() ? null : sysUserRpc.getById(item.getUpdateBy()).getData().getNickName());
            return copy;
        });
    }

    @Override
    public boolean tencentUserReport(List<Long> ids) {
        baseMapper.selectBatchIds(ids).stream()
                .filter(item -> !item.getIsBack().equals(BackStatusEnum.SUCCESS.getBackStatus()))
                .forEach(gameTencentUser -> {
                    String actionType = ActionTypeEnum.REGISTER.getActionType();
                    Map<String, Object> actionParam = new HashMap<>(2);
                    actionParam.put("claim_type", 0);
                    DataReportOfAppIdRpcDTO dataReportOfAppIdRpcDTO = DataReportOfAppIdRpcDTO.builder()
                            .appId(gameTencentUser.getWechatAppId())
                            .userActionSetId(gameTencentUser.getUserActionSetId())
                            .action(UserActionRpcDTO.builder()
                                    .actionTime(gameTencentUser.getRegisterTime())
                                    .actionType(actionType)
                                    .userId(UserActionRpcDTO.UserIdRpcDTO.builder()
                                            .wechatAppId(gameTencentUser.getWechatAppId())
                                            .wechatOpenid(gameTencentUser.getWechatOpenid())
                                            .build())
                                    .actionParam(actionParam)
                                    .build())
                            .build();

                    GameTencentBackLog gameTencentBackLog = GameTencentBackLog.builder()
                            .gameId(gameTencentUser.getGameId())
                            .adAccountId(gameTencentUser.getAdAccountId())
                            .actionTime(gameTencentUser.getRegisterTime())
                            .createTime(LocalDateTime.now())
                            .actionType(actionType)
                            .wechatAppId(gameTencentUser.getWechatAppId())
                            .wechatOpenid(gameTencentUser.getWechatOpenid())
                            .userActionSetId(gameTencentUser.getUserActionSetId())
                            .build();
                    gameTencentUser.setUpdateTime(LocalDateTime.now());
                    gameTencentUser.setUpdateBy(SecurityUtil.getUserId());
                    ResultVO<Boolean> result = userActionSetRpc.reportByAppId(dataReportOfAppIdRpcDTO);
                    if (result.getCode() == HttpStatus.HTTP_OK) {
                        gameTencentUser.setIsBack(BackStatusEnum.SUCCESS.getBackStatus());
                        gameTencentUser.setBackLog("回传成功");
                        updateById(gameTencentUser);

                        gameTencentBackLog.setBackLog("回传成功");
                        gameTencentBackLogService.save(gameTencentBackLog);
                    } else {
                        log.error("回传腾讯用户失败，失败原因：{}", result.getMsg());
                        gameTencentUser.setBackLog(result.getMsg());
                        gameTencentUser.setIsBack(BackStatusEnum.FAILED.getBackStatus());
                        updateById(gameTencentUser);

                        gameTencentBackLog.setBackLog(result.getMsg());
                        gameTencentBackLogService.saveOrUpdate(gameTencentBackLog);
                    }

                });
        return true;
    }
}
